import 'reflect-metadata';
import * as tdc from 'io-ts-derive-class';
declare type primitive = string | number | boolean | null | undefined | Date;
declare function isPrimitive(input: any): input is primitive;
declare type ValidatorResult = string | null | Promise<string | null>;
declare type GenericValidatorResult<T> = {
    [P in keyof T]?: T[P] extends primitive ? ValidatorResult : T[P] extends Array<infer U> ? ValidatorResult : never;
};
interface IValidator<T, Ctx> {
    (model: T, ctx: Ctx, originalModel?: T): ValidatorResult | GenericValidatorResult<T>;
}
interface IGenericValidator<T, Ctx> {
    (model: T, ctx: Ctx, originalModel?: T): GenericValidatorResult<T>;
}
export declare abstract class FieldValidator {
    readonly tag: string;
    abstract validate(value: any): ValidatorResult;
}
export declare type ValidationModel<T, Ctx> = {
    [P in keyof T]?: T[P] extends primitive ? FieldValidator | IValidator<T, Ctx> | Array<FieldValidator | IValidator<T, Ctx>> : T[P] extends Array<infer U> ? FieldValidator | IValidator<T, Ctx> | Array<FieldValidator | IValidator<T, Ctx>> : IGenericValidator<T, Ctx> | Array<IGenericValidator<T, Ctx>>;
};
declare type ValidatorEntryProps<T, Ctx> = {
    [P in keyof T]?: Array<FieldValidator | IValidator<T, Ctx>>;
};
declare type RequiredProps<T> = {
    [P in keyof T]: boolean;
};
declare class RequiredValidator extends FieldValidator {
    message: string;
    readonly isRequiredFieldValidator: boolean;
    constructor(message: string);
    validate(value: any): string | null;
}
declare class MinValidator extends FieldValidator {
    min: number;
    message: string | null;
    constructor(min: number, message?: string | null);
    validate(value: any): string | null;
}
declare class MaxValidator extends FieldValidator {
    max: number;
    message: string | null;
    constructor(max: number, message?: string | null);
    validate(value: any): string | null;
}
export declare const required: (message?: string) => RequiredValidator;
export declare const min: (min: number, message?: string | null) => MinValidator;
export declare const max: (max: number, message?: string | null) => MaxValidator;
declare type ValidationArray<T> = Array<T> & {
    errors: string[];
};
export interface IValidationRegistry<Ctx> {
    register<T>(klass: new (...args: any[]) => T, map: ValidationModel<T, Ctx>): void;
    validate<T extends tdc.ITyped<any>>(model: T, ctx: Ctx, originalModel?: T | undefined, validationPath?: string | null, path?: string): Promise<ValidationResult<T>>;
    getRequiredFieldsFor<T>(klass: new (...args: any[]) => T): RequiredProps<T>;
}
export declare class ValidationRegistry<Ctx> implements IValidationRegistry<Ctx> {
    readonly map: Map<any, any>;
    getValidatorsFor<T>(klass: new (...args: any[]) => T): ValidatorEntryProps<T, Ctx>;
    register<T>(klass: new (...args: any[]) => T, map: ValidationModel<T, Ctx>): void;
    validate<T extends tdc.ITyped<any>>(model: T, ctx: Ctx, originalModel?: T | undefined, validationPath?: string | null, path?: string): Promise<ValidationResult<T>>;
    getRequiredFieldsFor<T>(klass: new (...args: any[]) => T): RequiredProps<T>;
}
export declare function isValidationArray<T>(input: any): input is ValidationArray<T>;
export declare type ValidationResult<T> = {
    [P in keyof T]: T[P] extends primitive ? string[] : T[P] extends Array<infer U> ? ValidationArray<ValidationResult<U>> : ValidationResult<T[P]>;
};
declare function isInValidationPath(currentPath: string, validationPath: string | null): boolean;
export declare function isValid<T>(result: ValidationResult<T>): boolean;
declare function hasKey(input: any, key: string): boolean;
declare function isTypedRecord(input: any): boolean;
declare function isRecord(input: any): input is object;
export declare const _privates: {
    isPrimitive: typeof isPrimitive;
    isInValidationPath: typeof isInValidationPath;
    hasKey: typeof hasKey;
    isTypedRecord: typeof isTypedRecord;
    isRecord: typeof isRecord;
};
export {};
